#ifndef _compsat_h_INCLUDED
#define _compsat_h_INCLUDED

/*------------------------------------------------------------------------*/

#include <stdlib.h>		/* size_t */
#include <stdio.h>		/* FILE */

/*------------------------------------------------------------------------*/

#define COMPSAT_UNDECIDED	0	/* SAT 200[23] competition */
#define COMPSAT_SATISFIABLE	10	/* SAT 200[23] competition */
#define COMPSAT_UNSATISFIABLE	20 	/* SAT 200[23] competition */
#define COMPSAT_TIME_OUT	30
#define COMPSAT_SPACE_OUT	40
#define COMPSAT_DECISION_OUT	50
#define COMPSAT_ABORT		60
#define COMPSAT_EXIT		70

/*------------------------------------------------------------------------*/

typedef struct CompSat CompSat;
typedef struct COMPSAT_ArgvIterator COMPSAT_ArgvIterator;
typedef struct COMPSAT_ArgvIteratorInternal COMPSAT_ArgvIteratorInternal;

/*------------------------------------------------------------------------*/

struct COMPSAT_ArgvIterator
{
  COMPSAT_ArgvIteratorInternal *internal;
  const char *opt;		/* option string */
  const char *arg;		/* argument string or zero */
};

/*------------------------------------------------------------------------*/

CompSat *COMPSAT_new (void);
void COMPSAT_delete (CompSat *);
const char *COMPSAT_id (void);
const char *COMPSAT_version (void);
const char *COMPSAT_copyright (void);

/*------------------------------------------------------------------------*/

const char *COMPSAT_res2str (int res);
void COMPSAT_print_assignment (CompSat *, FILE *);
int COMPSAT_sat (CompSat *);

/*------------------------------------------------------------------------*/

int COMPSAT_is_file_option (const char *opt);
void COMPSAT_report_invalid_option (CompSat *,
				   COMPSAT_ArgvIterator * it, const char *err);
COMPSAT_ArgvIterator *COMPSAT_new_ArgvIterator (CompSat *, int, char **);
int COMPSAT_has_more_elements_ArgvIterator (COMPSAT_ArgvIterator *,
					   const char **);
void COMPSAT_delete_ArgvIterator (COMPSAT_ArgvIterator *);
const char *COMPSAT_usage (void);
const char *COMPSAT_option (CompSat * compsat, const char *opt, const char *arg);

/*------------------------------------------------------------------------*/

const char *COMPSAT_read (CompSat *, const char *file_name);

/*------------------------------------------------------------------------*/

void COMPSAT_print (CompSat *, FILE *);

double COMPSAT_get_time (void);
double COMPSAT_seconds (CompSat *);
size_t COMPSAT_bytes (CompSat *);
void COMPSAT_print_bytes (FILE *, size_t bytes);
void COMPSAT_print_seconds (FILE *, double seconds);
int COMPSAT_verbose_level (CompSat *);
const char *COMPSAT_prefix (CompSat *);
void COMPSAT_stats (CompSat *, FILE *);
FILE *COMPSAT_out_file (CompSat *);
FILE *COMPSAT_log_file (CompSat *);
FILE *COMPSAT_err_file (CompSat *);

/*------------------------------------------------------------------------*/
#endif
