#include "compsat.h"

/*------------------------------------------------------------------------*/

#include <signal.h>
#include <unistd.h>

/*------------------------------------------------------------------------*/

extern int COMPSAT_main (CompSat *, int, char **);

/*------------------------------------------------------------------------*/

static CompSat *compsat;

/*------------------------------------------------------------------------*/

static int
normalize_result (int res)
{
  if (res != COMPSAT_UNSATISFIABLE && res != COMPSAT_SATISFIABLE)
    res = COMPSAT_UNDECIDED;

  return res;
}

/*------------------------------------------------------------------------*/

static void
catch_signal (int sid)
{
  const char *prefix;
  FILE *file;
  int res;

  if (!compsat)
    exit (normalize_result (COMPSAT_ABORT));

  switch (sid)
    {
    case SIGINT:
      res = COMPSAT_ABORT;
      break;

    case SIGALRM:
      res = COMPSAT_TIME_OUT;
      break;

    default:
      res = COMPSAT_ABORT;
      break;
    }

  if (!COMPSAT_verbose_level (compsat))
    goto KILL_AND_EXIT;

  file = COMPSAT_log_file (compsat);
  prefix = COMPSAT_prefix (compsat);
  fprintf (file, "%s*** ", prefix);

  switch (sid)
    {
    case SIGINT:
      res = COMPSAT_ABORT;
      fprintf (file, "interrupt");
      break;

    case SIGALRM:
      res = COMPSAT_TIME_OUT;
      fprintf (file, "time out");
      break;

    default:
      res = COMPSAT_ABORT;
      fprintf (file, "caught signal(%d)", sid);
      break;
    }

  fputs (" ***\n", file);
  fflush (file);

  COMPSAT_stats (compsat, file);
  fflush (file);

KILL_AND_EXIT:
  kill (getpid (), sid);
  exit (normalize_result (res));
}

/*------------------------------------------------------------------------*/

int
main (int argc, char **argv)
{
  CompSat *tmp;
  int res;

  compsat = COMPSAT_new ();

  (void) signal (SIGINT, catch_signal);
  (void) signal (SIGALRM, catch_signal);

  res = COMPSAT_main (compsat, argc, argv);

  tmp = compsat;
  compsat = 0;
  COMPSAT_delete (tmp);

  return normalize_result (res);
}
